using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class CompassConfiguration
{

    public string? Destination { get; set; }
    public int? HoursBack { get; set; }
    public string? MonthPattern { get; set; }
    public string? Pattern { get; set; }
    public string? Source { get; set; }
    public string? YearPattern { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderCompassConfigurationSourceGenerationContext.Default.CompassConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, CompassConfiguration? configuration)
    {
        if (configuration?.Destination is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(CompassConfiguration _)
    {
    }

    private static Models.CompassConfiguration Get(CompassConfiguration? configuration)
    {
        Models.CompassConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.Destination is null) throw new NullReferenceException(nameof(configuration.Destination));
        if (configuration.HoursBack is null) throw new NullReferenceException(nameof(configuration.HoursBack));
        if (configuration.MonthPattern is null) throw new NullReferenceException(nameof(configuration.MonthPattern));
        if (configuration.Pattern is null) throw new NullReferenceException(nameof(configuration.Pattern));
        if (configuration.Source is null) throw new NullReferenceException(nameof(configuration.Source));
        if (configuration.YearPattern is null) throw new NullReferenceException(nameof(configuration.YearPattern));
        Verify(configuration);
        result = new(configuration.Destination,
                     configuration.HoursBack.Value,
                     configuration.MonthPattern,
                     configuration.Pattern,
                     configuration.Source,
                     configuration.YearPattern);
        return result;
    }

    public static Models.CompassConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.CompassConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.CompassConfiguration));
#pragma warning disable IL3050, IL2026
        CompassConfiguration? configuration = configurationSection.Get<CompassConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(CompassConfiguration))]
internal partial class BinderCompassConfigurationSourceGenerationContext : JsonSerializerContext
{
}