using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class SerialConfiguration
{

    public string? Destination { get; set; }
    public string? GhostPCLFileName { get; set; }
    public string? LincPDFCFileName { get; set; }
    public string? PortName { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderSerialConfigurationSourceGenerationContext.Default.SerialConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, SerialConfiguration? configuration)
    {
        if (configuration?.Destination is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(SerialConfiguration _)
    { }

    private static Models.SerialConfiguration Get(SerialConfiguration? configuration)
    {
        Models.SerialConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.Destination is null) throw new NullReferenceException(nameof(configuration.Destination));
        if (configuration.GhostPCLFileName is null) throw new NullReferenceException(nameof(configuration.GhostPCLFileName));
        if (configuration.LincPDFCFileName is null) throw new NullReferenceException(nameof(configuration.LincPDFCFileName));
        if (configuration.PortName is null) throw new NullReferenceException(nameof(configuration.PortName));
        Verify(configuration);
        result = new(configuration.Destination,
                     configuration.GhostPCLFileName,
                     configuration.LincPDFCFileName,
                     configuration.PortName);
        return result;
    }

    public static Models.SerialConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.SerialConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.SerialConfiguration));
#pragma warning disable IL3050, IL2026
        SerialConfiguration? configuration = configurationSection.Get<SerialConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(SerialConfiguration))]
internal partial class BinderSerialConfigurationSourceGenerationContext : JsonSerializerContext
{
}