using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class StratusConfiguration
{

    public string? Destination { get; set; }
    public string? FileDelimiterPattern { get; set; }
    public string? WatchFile { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderStratusConfigurationSourceGenerationContext.Default.StratusConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, StratusConfiguration? configuration)
    {
        if (configuration?.Destination is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(StratusConfiguration _)
    {
    }

    private static Models.StratusConfiguration Get(StratusConfiguration? configuration)
    {
        Models.StratusConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.Destination is null) throw new NullReferenceException(nameof(configuration.Destination));
        if (configuration.FileDelimiterPattern is null) throw new NullReferenceException(nameof(configuration.FileDelimiterPattern));
        if (configuration.WatchFile is null) throw new NullReferenceException(nameof(configuration.WatchFile));
        Verify(configuration);
        result = new(configuration.Destination,
                     configuration.FileDelimiterPattern,
                     configuration.WatchFile);
        return result;
    }

    public static Models.StratusConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.StratusConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.StratusConfiguration));
#pragma warning disable IL3050, IL2026
        StratusConfiguration? configuration = configurationSection.Get<StratusConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(StratusConfiguration))]
internal partial class BinderStratusConfigurationSourceGenerationContext : JsonSerializerContext
{
}