using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class TransmissionControlProtocolConfiguration
{

    public string? Destination { get; set; }
    public string? GhostPCLFileName { get; set; }
    public string? IPAddress { get; set; }
    public int? Port { get; set; }
    public bool? Server { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderTransmissionControlProtocolConfigurationSourceGenerationContext.Default.TransmissionControlProtocolConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, TransmissionControlProtocolConfiguration? configuration)
    {
        if (configuration?.Destination is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(TransmissionControlProtocolConfiguration _)
    {
        // if (transmissionControlProtocolConfiguration.IPAddresses is null) throw new NullReferenceException(nameof(transmissionControlProtocolConfiguration.IPAddresses));
        // foreach (KeyValuePair<string, string[]> keyValuePair in transmissionControlProtocolConfiguration.IPAddresses)
        // {
        //     if (string.IsNullOrEmpty(keyValuePair.Key))
        //         throw new Exception(nameof(IPAddresses));
        //     foreach (string value in keyValuePair.Value)
        //     {
        //         if (string.IsNullOrEmpty(value))
        //             throw new Exception(nameof(IPAddresses));
        //     }
        // }
    }

    private static Models.TransmissionControlProtocolConfiguration Get(TransmissionControlProtocolConfiguration? configuration)
    {
        Models.TransmissionControlProtocolConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.Destination is null) throw new NullReferenceException(nameof(configuration.Destination));
        if (configuration.GhostPCLFileName is null) throw new NullReferenceException(nameof(configuration.GhostPCLFileName));
        if (configuration.IPAddress is null) throw new NullReferenceException(nameof(configuration.IPAddress));
        if (configuration.Port is null) throw new NullReferenceException(nameof(configuration.Port));
        if (configuration.Server is null) throw new NullReferenceException(nameof(configuration.Server));
        Verify(configuration);
        result = new(configuration.Destination,
                     configuration.GhostPCLFileName,
                     configuration.IPAddress,
                     configuration.Port.Value,
                     configuration.Server.Value);
        return result;
    }

    public static Models.TransmissionControlProtocolConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.TransmissionControlProtocolConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.TransmissionControlProtocolConfiguration));
#pragma warning disable IL3050, IL2026
        TransmissionControlProtocolConfiguration? configuration = configurationSection.Get<TransmissionControlProtocolConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(TransmissionControlProtocolConfiguration))]
internal partial class BinderTransmissionControlProtocolConfigurationSourceGenerationContext : JsonSerializerContext
{
}