using System.Text.Json;
using System.Text.Json.Serialization;

namespace File_Watcher.Models.Binder;

public class WaferCounterConfiguration
{

    public string? Destination { get; set; }
    public bool? MatchPath { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, BinderWaferCounterConfigurationSourceGenerationContext.Default.WaferCounterConfiguration);
        return result;
    }

    private static void PreVerify(IConfigurationRoot configurationRoot, WaferCounterConfiguration? configuration)
    {
        if (configuration?.Destination is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
    }

    private static void Verify(WaferCounterConfiguration _)
    {
    }

    private static Models.WaferCounterConfiguration Get(WaferCounterConfiguration? configuration)
    {
        Models.WaferCounterConfiguration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.Destination is null) throw new NullReferenceException(nameof(configuration.Destination));
        if (configuration.MatchPath is null) throw new NullReferenceException(nameof(configuration.MatchPath));
        Verify(configuration);
        result = new(configuration.Destination,
                     configuration.MatchPath.Value);
        return result;
    }

    public static Models.WaferCounterConfiguration Get(IConfigurationRoot configurationRoot)
    {
        Models.WaferCounterConfiguration result;
        IConfigurationSection configurationSection = configurationRoot.GetSection(nameof(Models.WaferCounterConfiguration));
#pragma warning disable IL3050, IL2026
        WaferCounterConfiguration? configuration = configurationSection.Get<WaferCounterConfiguration>();
#pragma warning restore IL3050, IL2026
        PreVerify(configurationRoot, configuration);
        result = Get(configuration);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(WaferCounterConfiguration))]
internal partial class BinderWaferCounterConfigurationSourceGenerationContext : JsonSerializerContext
{
}